#!/usr/bin/env python
#-*- coding:utf-8 -*-


# import modules
import sys
import grass.script as grass


def main():
#def PopulateErrors():
    """
    check Oe/Ce for Pareto-optimal maps
    """


##############################################################################
### input:


    # grids from which to pull cats
    group_of_lowres_vector_grid_points_raw = grass.read_command('g.mlist',\
                                        type = "vect",\
                                        pat = "pareto_grid___OVER___*___ON___*_points")
    group_of_lowres_vector_grid_points = group_of_lowres_vector_grid_points_raw.splitlines()

    # thresholds integers
    thresholds_int = [1] + range(10,110,10)
    # thresholds_int = options['thresholds_int']

### output:
##############################################################################


    print " > Checking the Pareto-Optima maps for Omission/ Commission errors"


##############################################################################
# make a nice informative table about Omission/ Commission errors !?
    print " Omission errors: "
    width = 78
    print '-' * width
    print " a. ..."
    print " b. ..."

    # update control strings (value = accurate)
    print "\n  * Updating control key values and strings:"
    print "\n  > Accurate   = 0"
    print "  > Omission   = 1"
    print "  > Commission = 2\n"

##############################################################################


    # loop over lowres_vector_grid_points
    for lowres_vector_grid_points in group_of_lowres_vector_grid_points:


        print "\n  > Processing map %s" % lowres_vector_grid_points

        print "\n    * Control key values and strings updated for:"

        ### for loop ###
        for t in thresholds_int:

            # variables here
            control = "t%s_control" % t
            control_string = "t%s_control_string" % t
            erroneous_pix = "t%s_erroneous_pix" % t


        # Accurate (=agreement between reference and classified pixel):
        # control= 0 and control_string= accurate

            # Two /accurate/ possibilities:

            # "Reference = NULL"  and  "Classification = NULL"
            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control,\
            value = 0,\
            where = "coi_percent is NULL and t%d is NULL" % t)

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control_string,\
            value = "accurate",\
            where = "coi_percent is NULL and t%d is NULL" % t)

            # "Reference % Burned"  == "Classification is Burned"
            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control,\
            value = 0,\
            where = "total_pix == coi_pix")

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control_string,\
            value = "accurate",\
            where = "total_pix == coi_pix")


        # Omissions: control = 1  and  control_string = "omission" ; NOTE: t = 1% will have no omission
        # "Reference Burned %"  >  "Classified Burned %".
        # omission =

            # a. "Reference Burned %" is not NULL  and  "Classified Burned %" is NULL ==> omission = 100 %
            # NOTE: coi_percent >= 1

            # omission = burned pixels
            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control,\
            value = 1,\
            where ="coi_percent >= 1 and t%d is NULL" % t)

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control_string,\
            value = "omission",\
            where = "coi_percent >= 1 and t%d is NULL" % t)

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = erroneous_pix,\
            qcolumn = "coi_pix",\
            where = "coi_percent >= 1 and t%d is NULL" % t)


            # b. "Reference Burned %" (is not NULL)  >  "Classified Burned %" (is not NULL)

            # clarification:
                # Reference Burned == burned pixels == coi_pix
                # "reference_pix" == number of pixels within low-resolution gridcell

          # omission = burned pixels - threshold % of reference = coi_pix - reference / threshold %

    #      v.db.update lowres_vector_grid_points column=control value=1 where="coi_percent is not NULL and %s is NULL"
    #      v.db.update lowres_vector_grid_points column=control_string value="omission" where="coi_percent >= 1 and coi_percent > %s"
    #print " UPDATE lowres_vector_grid_points SET erroneous_pix = coi_pix WHERE lowres_vector_grid_points.coi_percent >= 1 AND lowres_vector_grid_points.coi_percent > lowres_vector_grid_points.%s " | db.execute


        # Commissions: control = 2  and  control_string = "commission" ; NOTE: t = 100% will have no commission
          # Two commission possibilities:

            # a. "Reference Burned %" is NULL  and "Classified Burned %" is not NULL (=100 %)
            # commission = reference pixels (=maximum number of centroids within the low-resolution grid-cell)

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control,\
            value = 2,\
            where = "coi_percent is NULL and t%d is not NULL" % t)

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control_string,\
            value = "commission",\
            where = "coi_percent is NULL and t%d is not NULL" % t)

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = erroneous_pix,\
            qcol = "total_pix",\
            where = "coi_percent is NULL and t%d is not NULL" % t)


            # b. "Reference Burned %" (is not NULL)  <  "Classified Burned %" (is not NULL)
            # commission = reference pixels - burned pixels

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control,\
            value = 2,\
            where = "coi_percent < 100 and coi_percent > t%d and t%d is not NULL" % (t, t))

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = control_string,\
            value = "commission",\
            where = "coi_percent < 100 and coi_percent > t%d and t%d is not NULL" % (t, t))

            grass.run_command('v.db.update',\
            map = lowres_vector_grid_points,\
            column = erroneous_pix,\
            value = "total_pix - coi_pix",\
            where = "coi_percent < 100 and coi_percent > t%d and t%d is not NULL" % (t, t))

            # how to make this print-out replicate a progress... (something like in gdal command line tools)
            print "    ",; sys.stdout.write("<t%d>.." % t)

        print"\n"

    print "\n>> Control key values and strings updated for pareto-optimal maps\n"


#  make it both a reusable module and a stand-alone utility
if __name__ == "__main__":
    options, flags = grass.parser()
    main()

